@@ -1,8 +1,8 @@
-# $Id: PropertyList.pm,v 1.3 2002/08/16 04:23:01 comdog Exp $
+# $Id: PropertyList.pm,v 1.5 2002/08/16 17:04:39 comdog Exp $
 package Mac::PropertyList;
 use strict;
 
-use vars qw($ERROR);
+use vars qw($ERROR $XML_head $XML_foot);
 
 =head1 NAME
 
@@ -161,8 +161,9 @@ sub parse_plist
 	$text =~ s|<true/>|<true>true</true>|g;
 	$text =~ s|<false/>|<false>false</false>|g;
 
+	# we can handle either 0.9 or 1.0
 	my @plist = extract_tagged( $text, 
-		'<plist version="1.0">', "</plist>",
+		'<plist version="\d.\d">', "</plist>",
 		 '(?s)<\?xml.*(?=<plist.*?>)', $Options );
 
 	if( $@ )
@@ -193,7 +194,6 @@ sub read_string  { _hash( 'string',  $_[0] ) }
 sub read_integer { _hash( 'integer', $_[0] ) }
 sub read_date    { _hash( 'date',    $_[0] ) }
 sub read_real    { _hash( 'real',    $_[0] ) }
-sub read_float   { _hash( 'float',   $_[0] ) }
 sub read_true    { _hash( 'true',    $_[0] ) }
 sub read_false   { _hash( 'false',   $_[0] ) }
 
@@ -253,13 +253,29 @@ UNIMPLEMENTED
 
 =cut
 
+$XML_head =<<"XML";
+<?xml version="1.0" encoding="UTF-8"?>
+<!DOCTYPE plist PUBLIC "-//Apple Computer//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
+<plist version="1.0">
+XML
+
+$XML_foot =<<"XML";
+</plist>
+XML
+
 sub plist_as_string
 	{
-	my $ref = shift;
+	my $hash = shift;
+
+	my( $type, $value ) = @{ $hash }{ qw( type value ) };
+
+	my $string = $XML_head;
 	
-	require Carp;
+	$string .= $Writers{$type}->($value) . "\n";
 	
-	carp( "plist_as_string is unimplemented" );
+	$string .= $XML_foot;
+	
+	return $string;
 	}
 	
 sub _string { "<$_[0]>$_[1]</$_[0]>" }
@@ -268,11 +284,10 @@ sub write_string  { _string( 'string',  $_[0] ) }
 sub write_integer { _string( 'integer', $_[0] ) }
 sub write_date    { _string( 'date',    $_[0] ) }
 sub write_real    { _string( 'real',    $_[0] ) }
-sub write_float   { _string( 'float',   $_[0] ) }
 sub write_true    { "<true/>" }
 sub write_false   { "<false/>" }
 
-sub write_data
+sub write_data($)
 	{
 	my $string = shift;
 	
@@ -285,10 +300,7 @@ sub write_data
 
 sub write_array
 	{
-	my $hash = shift;
-	
-	my( $type, $array ) = @{ $hash }{ qw( type value ) };
-	return unless $type eq 'array';
+	my $array = shift;
 	
 	my $string = "<array>\n";
 	
@@ -298,22 +310,19 @@ sub write_array
 		
 		my $bit = $Writers{$type}->( $value );
 		
-		$bit =~ s/$/\t/gm;
+		$bit =~ s/^/\t/gm;
 		
 		$string .= $bit . "\n";
 		}
 	
-	$string .= "</array>\n";
+	$string .= "</array>";
 	
 	return $string;		
 	}
 
 sub write_dict
 	{
-	my $hash = shift;
-	
-	my( $type, $dict ) = @{ $hash }{ qw( type value ) };
-	return unless $type eq 'dict';
+	my $dict  = shift;
 	
 	my $string = "<dict>\n";
 	
@@ -324,12 +333,12 @@ sub write_dict
 		my $bit  = _string( 'key', $key ) . "\n";
 		   $bit .= $Writers{$type}->( $value ) . "\n";
 		
-		$bit =~ s/$/\t/gm;
+		$bit =~ s/^/\t/gm;
 		
 		$string .= $bit;
 		}
 	
-	$string .= "</dict>\n";
+	$string .= "</dict>";
 	
 	return $string;		
 	}
